/*
* This file is part of Converseen, an open-source batch image converter
* and resizer.
*
* (C) Francesco Mondello 2009-2012
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
* Contact e-mail: Faster <faster3ck@gmail.com>
*
*/

#ifndef MAINWINDOWIMPL_H
#define MAINWINDOWIMPL_H
//
#include <QMainWindow>
#include <QtGui>
#include <QFileInfo>
#include <QList>
#include <QSettings>
#include "ui_mainwindow.h"
#include "converter.h"
#include "dialogquality.h"
#include "dialogconversionstatus.h"
//
class ImageAttributes
{
public:
    QString completeFileName, fileName, suffix, path, format;
    qint64 size;
};
//
class MainWindowImpl : public QMainWindow, public Ui::MainWindow
{
    Q_OBJECT
public:
    MainWindowImpl( QWidget * parent = 0, Qt::WFlags f = 0 );
    void dragEnterEvent(QDragEnterEvent *event);
    void dropEvent(QDropEvent *event);
    void importListFromArgv(QString fileName);

protected:
    void resizeEvent(QResizeEvent *event);

private:
    void createActions();
    void setupMenu();
    void createContextMenu();
    void setFileInList(QString fileName);
    void loadFiles(QStringList fileNames);
    bool checkDuplicates(QStringList fileNames, int e);
    void convert();
    void fillTreeView();
    void loadFormats();
    void loadOptions();
    void loadQuality();

    void deleteItems(int selectedItems);

    QImage *toQImage(const Image &image);
    void loadPreview(QString fileName);
    void showImageInformations();   // quello che succede alla selezione dell'item.

    bool overwriteMode();
    QString simplifyFileSize(qint64 size);
    bool areThereImagesChecked();   // Verify if there are images checked
    void resetDisplays();

    QSettings *settings;

    QList<ImageAttributes> iAList;
    QString readableFiltersString;
    QStringList readableFomats;

    QAction *actionOpenFiles;
    QAction *actionAddFiles;
    QAction *actionShowOptions;
    QAction *actionClose;

    QAction *actionConvert;

    QAction *actionCheckItems;
    QAction *actionCheckAllItems;
    QAction *actionUncheckItems;
    QAction *actionUncheckAllItems;
    QAction *actionRemoveItems;
    QAction *actionRemoveAllItems;
    QAction *actionInfo;

    Converter *convertThread;
    DialogConversionStatus *dlgCStatus;
    int curr_index;             // Indice dell'immagine che si sta processando dalla lista iAList
    QString out_format;
    QString out_path;
    int quality;

    int jpgQuality;             // Imposta la qualità jpg anche nel file ini
    int pngQuality;             // Imposta la qualità png anche nel file ini
    int img_width,img_height;  /* vengono riempiti solo quando un'immagine è selezionata.
                                  Riempire con dati su ImageAttributes potrebbe essere lento e non conveniente! */
    int new_img_width,new_img_height; /* Vengono riempiti con numeri che rappresentano i pixel dato che non capisco come
                                         si passa la corretta percentuale in virgola mobile a geometry*/
    int m_xResolution, m_yResolution;

    bool abort_all;
private slots:
    void openFiles();
    void addFiles();
    void elabora();
    void setCurrentDirectory();
    void openOutDirectory();
    void checkItems();
    void checkAllItems();
    void uncheckItems();
    void uncheckAllItems();
    void removeItems();
    void removeAllItems();
    void setQuality();
    void selectGeometryUnit(QString unit);
    void relativeSizeW(double value);
    void relativeSizeH(double value);

    void onItemSelection();
    void editSettings();

    void about();

    void enableRenameLine();
    void enableShowPreview();

    void showNewSizePreview();

    void onPushResetClick();

    void startConversion();
    void nextConversion(int conv_status);

    void overwriteDialog(QString baseName);
    void saveOverwritingSettings();

    void stopProcess();
    void errorMessage(QString err_status);
};
#endif // MAINWINDOWIMPL_H
